@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.1.0 $
+### $Release: 0.2.0 $
 ### $Copyright: copyright(c) 2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -13,7 +13,7 @@ sys.STDERR = sys.stderr
 python2 = sys.version_info[0] == 2
 python3 = sys.version_info[0] == 3
 
-__version__ = "$Release: 0.1.0 $".split(' ')[1]
+__version__ = "$Release: 0.2.0 $".split(' ')[1]
 __unittest = 1
 DEBUG = False
 TEST = os.getenv("TEST")
@@ -96,6 +96,12 @@ class TestBuilder(object):
     def after(self, func):
         self._topic.after = func
 
+    def before_all(self, func):
+        self._topic.before_all = func
+
+    def after_all(self, func):
+        self._topic.after_all = func
+
     def fixture(self, gfunc):
         self._topic.fixtures[gfunc.__name__] = gfunc
 
@@ -128,12 +134,12 @@ class TestCallback(object):
 
 class TestTopic(object):
 
+    before = after = before_all = after_all = None
+
     def __init__(self, name, parent=None):
         self.name = name
         self.parent = parent
         self.children = []
-        self.before = None
-        self.after = None
         self.fixtures = {}
 
     def add_topic(self, topic):
@@ -185,6 +191,7 @@ class TestRunner(object):
         self._topics_stack.append(topic)
         self._topics_revstack.insert(0, topic)
         if not root_p: self.reporter.enter_topic(depth, topic.name)
+        if topic.before_all: topic.before_all()
         try:
             for child in topic.children:
                 #if isinstance(child, TestTopic):
@@ -194,6 +201,7 @@ class TestRunner(object):
                     desc, func = child
                     self._run_spec(depth+1, desc, func)
         finally:
+            if topic.after_all: topic.after_all()
             if not root_p: self.reporter.exit_topic(depth, topic.name)
             self._topics_stack.pop()
             self._topics_revstack.pop(0)
@@ -249,12 +257,12 @@ class TestRunner(object):
                 if gfunc:
                     g = _call_with_self(gfunc, self_)
                     if not isinstance(g, types.GeneratorType):
-                        ex = TypeError("fixture should yield value.")
+                        ex = TypeError("%s(): fixture should yield value instead of return." % (gfunc.__name__,))
                         break
                     arg = next(g)
                     generators.append(g)
                 else:
-                    ex = NameError("%r: corrensponding fixture not defined." % (argname,))
+                    ex = NameError("%s: corresponding fixture not found." % (argname,))
                     break
             args.append(arg)
         return args, generators, ex
@@ -498,13 +506,66 @@ def assertTextEqual(self, text1, text2):
     if text1 != text2:
         lines1 = text1.splitlines(True)
         lines2 = text2.splitlines(True)
+        if not lines1[-1].endswith("\n"):
+            lines1[-1] += "\n\\ No newline at end of text\n"
+        if not lines2[-1].endswith("\n"):
+            lines2[-1] += "\n\\ No newline at end of text\n"
         lst = [d for d in difflib.unified_diff(lines1, lines2, "expected", "actual")]
-        msg = "texts are not equal.\n" + ''.join(lst)
+        s = "".join(lst)
+        s = re.sub(r'^(\-\-\- expected) ?\n(\+\+\+ actual) ?\n', "\\1\n\\2\n", s, 1)
+        msg = "texts are not equal.\n" + s
     assert text1 == text2, msg
 
 unittest.TestCase.assertTextEqual = assertTextEqual
 del assertTextEqual
 
+def _classname(klass):
+    if isinstance(klass, type):
+        return klass.__name__
+    elif hasattr(types, "ClassType") and isinstance(klass, types.ClassType):
+        return klass.__name__
+    else:
+        return repr(klass)
+
+def assertException(self, fn, exc_class, errmsg=None):
+    if not isinstance(fn, types.FunctionType):
+        raise TypeError("First argument of assertException() should be a function")
+    ex = None
+    try:
+        fn()
+    except Exception:
+        ex = sys.exc_info()[1]
+    else:
+        self.fail("%s expected but nothing raised" % (_classname(exc_class),))
+    fn.exception = ex
+    if not isinstance(ex, exc_class):
+        self.fail("%s expected but got %r" % (_classname(exc_class), ex))
+    if errmsg is not None:
+        if isinstance(errmsg, _rexp_type):
+            matched = errmsg.search(str(ex))
+        else:
+            matched = str(ex) == errmsg
+        if not matched:
+            self.fail("%r expected but got %r" % (errmsg, str(ex)))
+    return ex
+
+unittest.TestCase.assertException = assertException
+del assertException
+_rexp_type = type(re.compile("x"))
+
+def assertNotException(self, fn, exc_class=Exception):
+    if not isinstance(fn, types.FunctionType):
+        raise TypeError("First argument of assertNotException() should be a function")
+    try:
+        fn()
+    except Exception:
+        ex = sys.exc_info()[1]
+        if isinstance(ex, exc_class):
+            self.fail("%s should not be raised but %r raised" % (_classname(exc_class), ex,))
+
+unittest.TestCase.assertNotException = assertNotException
+del assertNotException
+
 
 ##
 ## main
