@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.1.0 $
+### $Release: 0.2.0 $
 ### $Copyright: copyright(c) 2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -28,8 +28,8 @@ def _run(command):
     status_code = p.returncode
     return out, err, status_code
 
-def _run_picotest(input, encoding):
-    filename = "_test_pico.py"
+def _run_testscript(input, encoding):
+    filename = "_test_ex1.py"
     try:
         f = open(filename, "w"); f.write(input); f.close()
         command = "%s %s" % (sys.executable, filename)
@@ -59,12 +59,165 @@ def colorize(text):
     text = text.replace("</Y>", "\x1b[0m")
     return text
 
-def monolize(text):
+def monochromize(text):
     return re.sub(r'</?[bRGY]>', "", text)
 
 import picotest
 
 
+class TestCaseHelper_TC(unittest.TestCase):
+
+    ## assertTextEqual
+
+    def test_assertTextEqual_shows_unified_diff(self):
+        expected = "Haruhi\nMikuru\nYuki\n"
+        actual = "Haruhi\nMichiru\nYuki\n"
+        try:
+            self.assertTextEqual(expected, actual)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+        errmsg = r"""
+texts are not equal.
+--- expected
++++ actual
+@@ -1,3 +1,3 @@
+ Haruhi
+-Mikuru
++Michiru
+ Yuki
+"""[1:]
+        self.assertEqual(errmsg, str(ex))
+
+    def test_assertTextEqual_shows_No_newline_at_the_end_of_text(self):
+        expected = "Haruhi\nMikuru\nYuki\n"
+        actual = "Haruhi\nMikuru\nYuki"
+        try:
+            self.assertTextEqual(expected, actual)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+        errmsg = r"""
+texts are not equal.
+--- expected
++++ actual
+@@ -1,3 +1,3 @@
+ Haruhi
+ Mikuru
+-Yuki
++Yuki
+\ No newline at end of text
+"""[1:]
+        self.assertTextEqual(errmsg, str(ex))
+
+    ## assertException
+
+    def test_assertException_passes_when_expected_exception_raised(self):
+        def fn(): raise ValueError("SOS")
+        self.assertException(fn, ValueError)   # passed
+        self.assertTrue(isinstance(fn.exception, ValueError))
+        self.assertEqual("SOS", str(fn.exception))
+        #
+        def fn(): raise ValueError("SOS")
+        self.assertException(fn, ValueError, "SOS")   # passed
+        self.assertTrue(isinstance(fn.exception, ValueError))
+        self.assertEqual("SOS", str(fn.exception))
+
+    def test_assertException_fails_when_nothing_raised(self):
+        def fn(): pass
+        try:
+            self.assertException(fn, ValueError)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("ValueError expected but nothing raised", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised")
+
+    def test_assertException_fails_when_unexpected_exception_raised(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertException(fn, NameError)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("NameError expected but got ValueError('SOS',)", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    def test_assertException_fails_when_error_message_is_different(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertException(fn, ValueError, "SOS!")
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("'SOS!' expected but got 'SOS'", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    def test_assertException_can_take_regexp_as_error_message_pattern(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertException(fn, ValueError, re.compile(r'^[A-Z]+$'))
+        except Exception:
+            self.fail("*** unexpected exception raised: %r" % sys.exc_info()[1])
+        #
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertException(fn, ValueError, re.compile(r'^[a-z]+$'))
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertTrue(re.match(r"^<_sre\.SRE_Pattern object at 0x[0-9a-f]+> expected but got 'SOS'$", str(ex)))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    def test_assertException_raises_TypeErro_when_first_argument_is_not_function(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertException(ValueError, "SOS")
+        except TypeError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("First argument of assertException() should be a function", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    ## assertNotException
+
+    def test_assertNotException_passes_when_nothing_raised(self):
+        def fn(): pass
+        self.assertNotException(fn)   # pass
+
+    def test_assertNotException_passes_when_unexpected_exception_raised(self):
+        def fn(): raise ValueError("SOS")
+        self.assertNotException(fn, NameError)   # pass
+
+    def test_assertNotException_failed_when_something_raised(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertNotException(fn)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("Exception should not be raised but ValueError('SOS',) raised", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    def test_assertNotException_failed_when_expected_exception_raised(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertNotException(fn, ValueError)
+        except AssertionError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("ValueError should not be raised but ValueError('SOS',) raised", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+    def test_assertNotException_raises_TypeErro_when_first_argument_is_not_function(self):
+        def fn(): raise ValueError("SOS")
+        try:
+            self.assertNotException(ValueError)
+        except TypeError:
+            ex = sys.exc_info()[1]
+            self.assertEqual("First argument of assertNotException() should be a function", str(ex))
+        else:
+            self.fail("*** AssertionError expected but not raised.")
+
+
 class TestReporter_TC(unittest.TestCase):
 
     INPUT = r"""
@@ -126,12 +279,12 @@ if __name__ == "__main__":
     EXPECTED_ERROR = r"""
 ----------------------------------------------------------------------
 [<R>Failed</R>] ClassName > #method1() > 1-1 should be 0
-  File "_test_pico.py", line 17, in _
+  File "_test_ex1.py", line 17, in _
     assert 1-1 == 2
 AssertionError
 ----------------------------------------------------------------------
 [<R>ERROR</R>] ClassName > #method1() > 1/1 should be infinite
-  File "_test_pico.py", line 22, in _
+  File "_test_ex1.py", line 22, in _
     foobarbaz
 NameError: global name 'foobarbaz' is not defined
 ----------------------------------------------------------------------
@@ -171,13 +324,13 @@ AssertionError: Expected to be failed (because not implemented yet), but passed
     def _do_test(self, input, expected, encoding='utf-8'):
         if with_statement_not_available:
             input = _convert_with_statement_into_for_statement(input)
-        out, err, status_code = _run_picotest(input, encoding)
+        out, err, status_code = _run_testscript(input, encoding)
         self.assertTextEqual("", err)
         self.assertTextEqual(expected, out)
 
     def test_verbose_style_in_mono(self):
         input = self.INPUT % {'STYLE': "verbose", 'COLOR': False}
-        expected = monolize(self.EXPECTED_VERBOSE)
+        expected = monochromize(self.EXPECTED_VERBOSE)
         self._do_test(input, expected)
 
     def test_verbose_style_in_color(self):
@@ -189,12 +342,12 @@ AssertionError: Expected to be failed (because not implemented yet), but passed
         input = self.INPUT % {'STYLE': "verbose", 'COLOR': True}
         input = re.sub(r'(if __name__ == "__main__":\n)(.|\n)*',
                        '\\1    picotest.main("-s", "verbose")\n', input)
-        expected = "#### _test_pico.py\n" + colorize(self.EXPECTED_VERBOSE)
+        expected = "#### _test_ex1.py\n" + colorize(self.EXPECTED_VERBOSE)
         self._do_test(input, expected)
 
     def test_simple_style_in_mono(self):
         input = self.INPUT % {'STYLE': "simple", 'COLOR': False}
-        expected = monolize(self.EXPECTED_SIMPLE)
+        expected = monochromize(self.EXPECTED_SIMPLE)
         self._do_test(input, expected)
 
     def test_simple_style_in_color(self):
@@ -202,16 +355,16 @@ AssertionError: Expected to be failed (because not implemented yet), but passed
         expected = colorize(self.EXPECTED_SIMPLE)
         self._do_test(input, expected)
 
-    def test_simple_style_with_main(self):
+    def test_simple_style_with_mono(self):
         input = self.INPUT % {'STYLE': "simple", 'COLOR': True}
         input = re.sub(r'(if __name__ == "__main__":\n)(.|\n)*',
                        '\\1    picotest.main("-s", "simple")\n', input)
-        expected = "_test_pico.py: " + colorize(self.EXPECTED_SIMPLE)
+        expected = "_test_ex1.py: " + colorize(self.EXPECTED_SIMPLE)
         self._do_test(input, expected)
 
     def test_plain_style_in_mono(self):
         input = self.INPUT % {'STYLE': "plain", 'COLOR': False}
-        expected = monolize(self.EXPECTED_PLAIN)
+        expected = monochromize(self.EXPECTED_PLAIN)
         self._do_test(input, expected)
 
     def test_plain_style_in_color(self):
@@ -219,7 +372,7 @@ AssertionError: Expected to be failed (because not implemented yet), but passed
         expected = colorize(self.EXPECTED_PLAIN)
         self._do_test(input, expected)
 
-    def test_plain_style_with_main(self):
+    def test_plain_style_with_mono(self):
         input = self.INPUT % {'STYLE': "plain", 'COLOR': True}
         input = re.sub(r'(if __name__ == "__main__":\n)(.|\n)*',
                        '\\1    picotest.main("-s", "plain")\n', input)
@@ -232,7 +385,7 @@ class TestRunner_TC(unittest.TestCase):
     def _do_test(self, input, expected, encoding='utf-8'):
         if with_statement_not_available:
             input = _convert_with_statement_into_for_statement(input)
-        out, err, status_code = _run_picotest(input, encoding)
+        out, err, status_code = _run_testscript(input, encoding)
         self.assertTextEqual("", err)
         self.assertTextEqual(expected, out)
 
@@ -389,12 +542,79 @@ if __name__ == "__main__":
     - [ERROR] test#2
 ----------------------------------------------------------------------
 [Failed] Parent > Child > test#1
-  File "_test_pico.py", line 18, in _
+  File "_test_ex1.py", line 18, in _
     def _(self):  assert 1+1 == 0
 AssertionError
 ----------------------------------------------------------------------
 [ERROR] Parent > Child > test#2
-  File "_test_pico.py", line 21, in _
+  File "_test_ex1.py", line 21, in _
+    def _(self):  raise ValueError("failed")
+ValueError: failed
+----------------------------------------------------------------------
+## total:2, passed:0, failed:1, error:1, skipped:0, todo:0
+"""[1:]
+        self._do_test(input, expected)
+
+    def test_before_all_and_after_all_are_called_only_once(self):
+        input = r"""
+from __future__ import with_statement
+import picotest
+test = picotest.new()
+
+with test("Parent"):
+  @test.before_all
+  def _():      print("!!! Parent: before_all() called")
+  @test.after_all
+  def _():      print("!!! Parent: after_all() called")
+  @test.before
+  def _(self):  print("!!! Parent: before() called")
+  @test.after
+  def _(self):  print("!!! Parent: after() called")
+  #
+  with test("Child"):
+    @test.before_all
+    def _():      print("!!!   Child: before_all() called")
+    @test.after_all
+    def _():      print("!!!   Child: after_all() called")
+    @test.before
+    def _(self):  print("!!!   Child: before() called")
+    @test.after
+    def _(self):  print("!!!   Child: after() called")
+    #
+    @test("test#1")
+    def _(self):  assert 1+1 == 0
+    #
+    @test("test#2")
+    def _(self):  raise ValueError("failed")
+
+if __name__ == "__main__":
+  test.run()
+"""[1:]
+        expected = r"""
+* Parent
+!!! Parent: before_all() called
+  * Child
+!!!   Child: before_all() called
+!!! Parent: before() called
+!!!   Child: before() called
+!!!   Child: after() called
+!!! Parent: after() called
+    - [Failed] test#1
+!!! Parent: before() called
+!!!   Child: before() called
+!!!   Child: after() called
+!!! Parent: after() called
+    - [ERROR] test#2
+!!!   Child: after_all() called
+!!! Parent: after_all() called
+----------------------------------------------------------------------
+[Failed] Parent > Child > test#1
+  File "_test_ex1.py", line 26, in _
+    def _(self):  assert 1+1 == 0
+AssertionError
+----------------------------------------------------------------------
+[ERROR] Parent > Child > test#2
+  File "_test_ex1.py", line 29, in _
     def _(self):  raise ValueError("failed")
 ValueError: failed
 ----------------------------------------------------------------------
@@ -425,7 +645,7 @@ if __name__ == "__main__":
   - [ERROR] when fixture not found
 ----------------------------------------------------------------------
 [ERROR] Example > when fixture not found
-NameError: 'haruhi': corrensponding fixture not defined.
+NameError: haruhi: corresponding fixture not found.
 ----------------------------------------------------------------------
 ## total:1, passed:0, failed:0, error:1, skipped:0, todo:0
 """[1:]
@@ -516,7 +736,7 @@ if __name__ == "__main__":
     - [ERROR] fixtures are scoped
 ----------------------------------------------------------------------
 [ERROR] ClassName > #method2() > fixtures are scoped
-NameError: 'member': corrensponding fixture not defined.
+NameError: member: corresponding fixture not found.
 ----------------------------------------------------------------------
 ## total:1, passed:0, failed:0, error:1, skipped:0, todo:0
 """[1:]
@@ -545,7 +765,7 @@ if __name__ == "__main__":
   - [ERROR] fixture should yield
 ----------------------------------------------------------------------
 [ERROR] Example > fixture should yield
-TypeError: fixture should yield value.
+TypeError: team(): fixture should yield value instead of return.
 ----------------------------------------------------------------------
 ## total:1, passed:0, failed:0, error:1, skipped:0, todo:0
 """[1:]
@@ -659,7 +879,7 @@ if __name__ == "__main__":
     - [passed] fixture injection is available
 ----------------------------------------------------------------------
 [Failed] ClassName > #method1() > 1-1 should be 0
-  File "_test_pico.py", line 20, in _
+  File "_test_ex1.py", line 20, in _
     assert 1-1 == 2    # failed
 AssertionError
 ----------------------------------------------------------------------
